import os,sys   
abs_path = os.path.abspath(sys.argv[0])
basename = os.path.basename(sys.argv[0])
dirname = abs_path.replace(basename, "")
if not "%s/diskmon/lib" % dirname in sys.path:
   sys.path.append("%s/diskmon/lib" % dirname) # always with regard to parent script
if not "/usr/bp/lib/monitoring" in sys.path:
   sys.path.append("/usr/bp/lib/monitoring")
#from util      import *   # include config << bpdb as well 

# Add this to avoid dependency of util
def toHex(dec):
   x = (dec % 16)
   digits = "0123456789ABCDEF"
   rest = dec / 16
   if (rest == 0):
      return digits[x]
   return toHex(rest) + digits[x]

#################class Device ##########################
class Device:
   """Holding partition information from smartctl"""
   def __init__(self, index, model, family, thrput, cap, rpm, ss, isEnterprise=False, isWTBMeaningful=False):
      self.index  = index
      self.model  = model
      self.family = family
      
      self.vender = model[0:2]
      if (model[0:2] == "ST"):
         self.vender = "SG"
         if (model[0:3] == "STM"):
            self.vender = "MAXT"
      elif (model[0:2] == "HD"):
         self.vender = "HGST"

      self.thrput   = thrput
      self.hex      = toHex(index)
      if len(self.hex)==1:
         self.hex ="0%s"% self.hex
      self.capacity = cap
      self.rpm      = rpm
      self.sectorSize = ss
      self.ETC = isEnterprise
      self.WTB = isWTBMeaningful


   def __repr__(self):
      return "%3d" % self.index + "  0x%2s" % self.hex + "  %4s" % self.vender + " %34s" % self.family + "%26s" % self.model \
            + "  %4dGB" % self.capacity + "  %5drpm" % self.rpm + " %dGs" % self.thrput + "  %4dB" % self.sectorSize + " %s" % self.ETC 
   

def list_of_device(doPrint=False):
   list = [] #up to 255 for packing into 2^8 00-FF
   list.append(Device(  0, "ST3160815A",         "Barracuda 7200.10",      3,  160,  7200,  512)) #0x00
   list.append(Device(  1, "ST3250620AS",        "Barracuda 7200.10",      3,  250,  7200,  512)) #0x01
   list.append(Device(  2, "ST3250820AS",        "Barracuda 7200.10",      3,  250,  7200,  512)) #0x01
   list.append(Device(  3, "HDS722525VLSA80",    "Hitachi Deskstar 7K250", 3,  250,  7200,  512))
   list.append(Device(  4, "STM3500630AS",       "DiamondMax 21",          3,  500,  7200,  512))
   list.append(Device(  5, "ST3500320AS",        "Barracuda 7200.11",      3,  500,  7200,  512))
   list.append(Device(  6, "ST3500418AS",        "Barracuda 7200.12",      3,  500,  7200,  512))
   list.append(Device(  7, "ST3500630AS",        "Barracuda 7200.10",      3,  500,  7200,  512))
   list.append(Device(  8, "ST3600057SS",        "Cheetah 15K.7",          3,  600, 15000,  512))
   list.append(Device(  9, "ST3750330AS",        "Barracuda 7200.11",      3,  750,  7200,  512))
   list.append(Device( 10, "ST3750528AS",        "Barracuda 7200.12",      3,  750,  7200,  512))
   list.append(Device( 11, "ST3750525AS",        "Barracuda 7200.12",      6,  750,  7200,  512))
   list.append(Device( 12, "ST3750640AS",        "Barracuda 7200.10",      3,  750,  7200,  512))
   list.append(Device( 13, "HDT722516DLAT80",    "Hitachi GST Deskstar",   3,  160,  7200,  512)) #0x00
   list.append(Device( 14, "ST3500641AS",        "Barracuda 7200.9",       3,  500,  7200,  512)) #0x00
   list.append(Device( 15, "HDP725025GLAT80",    "Hitachi Deskstar 7K500", 3,  250,  7200,  512))
   #list.append(Device( 16, "WD2500AAJB-00J3A0",  "Western Digital Caviar SE family", 3,  250,  7200,  512))
   list.append(Device( 17, "ST3160215A",         "Barracuda",              3,  160,  7200,  512)) #0x00

   list.append(Device( 19, "INTEL_SSDSC2BW120A4",      "Intel",                  6,  120, 99999,  512)) #0x14
   list.append(Device( 20, "INTEL_SSDSC2CW120A3",      "Intel",                  6,  120, 99999,  512)) #0x14
   list.append(Device( 21, "INTEL_SSDSC2CW480A3",      "Intel",                  6,  480, 99999,  512))
   list.append(Device( 22, "ST750LX003",         "Momentus XT",            6,  750,  7200,  512))
   list.append(Device( 23, "ST900MM0026",        "SEAGATE",                6,  900, 10000,  512))
   list.append(Device( 24, "Crucial_CT128MX100SSD1",        "Crucial",     6,  128, 99999,  512))
   list.append(Device( 25, "Crucial_CT256MX100SSD1",        "Crucial",     6,  256, 99999,  512))
   list.append(Device( 26, "Micron_M600_MTFDDAK128MBF",     "Micron",      6,  128, 99999,  4096))
   list.append(Device( 27, "Micron_M600_MTFDDAK256MBF",     "Micron",      6,  256, 99999,  4096))
   list.append(Device( 28, "Micron_1100_MTFDDAK256TBN",     "Micron",      6,  256, 99999,  512))
   list.append(Device( 29, "KINGSTON_SKC400S37128G",        "Kingston",    6,  128, 99999,  512))
   list.append(Device( 30, "Samsung_SSD_845DC_EVO_480GB",        "Samsung",     6,  480, 99999,  512))
   list.append(Device( 31, "SAMSUNG_MZ7LM480HCHP-0E003",         "Samsung",     6,  480, 99999,  512))
   
   list.append(Device(100, "HDS721010KLA330",    "Hitachi",                3, 1000,  7200,  512)) #0x64
   list.append(Device(101, "ST31000340AS",       "Barracuda 7200.11",      3, 1000,  7200,  512)) #0x65
   list.append(Device(102, "ST31000524AS",       "Barracuda 7200.12",      6, 1000,  7200,  512))
   list.append(Device(103, "ST31000528AS",       "Barracuda 7200.12",      3, 1000,  7200,  512)) #this is 3G/s version
   list.append(Device(104, "ST1000DM003-9YN162", "Barracuda",              6, 1000,  7200, 4096)) #0x68
   list.append(Device(105, "ST1000DM003-1CH162", "Barracuda",              6, 1000,  7200, 4096, False, True)) #0x69
   list.append(Device(107, "ST1000NX0333",       "Barracuda 2.5 SAS",      6, 1000,  7200,  512, True)) #0x6B
   list.append(Device(108, "WDC_WD10JFCX-68N6GN0", "RE  Serial ATA",         6, 1000,  5900,  512, True)) #0x6C
   list.append(Device(109, "WDC_WD1004FBYZ-01YCBB1", "RE  Serial ATA",         6, 1000,  7200,  512, True)) #0x6D
   list.append(Device(110, "WDC_WD1000CHTZ-04JCPV0", "VelociRaptor",           6, 1000, 10000, 4096, True)) #0x6E
   list.append(Device(111, "WDC_WD1003FBYX-01Y7B1",  "RE4 Serial ATA 01.01V02", 3, 1000,  7200,  512, True)) #0x6F
   list.append(Device(112, "WDC_WD1003FBYX-01Y7B0",  "RE4 Serial ATA 01.01V01", 3, 1000,  7200,  512, True)) #0x70
   list.append(Device(113, "WDC_WD1003FBYZ-010FB0",  "RE  Serial ATA",          6, 1000,  7200,  512, True)) #0x71
   list.append(Device(114, "WDC_WD1003FBYX-01Y7B2",  "RE4 Serial ATA 01.01V03", 3, 1000,  7200,  512, True)) #0x72
   list.append(Device(115, "ST31500341AS",       "Barracuda 7200.11",      3, 1500,  7200,  512)) #0x73
   list.append(Device(116, "ST1500DM003-9YN16G", "Barracuda",              6, 1500,  7200, 4096)) #0x74
   list.append(Device(117, "ST1500DM003-1CH16G", "Barracuda",              6, 1500,  7200, 4096, False, True)) #0x75
   list.append(Device(118, "ST1000NM0033-9ZM173",  "Enterprise Constellation ES3", 6, 1000,  7200,  512, True)) #0x76
   list.append(Device(119, "WDC_WD1000CHTZ-04JCPV1", "VelociRaptor",           6, 1000, 10000, 4096, True)) #0x6E
   list.append(Device(120, "ST32000641AS",       "Barracuda XT",           3, 2000,  7200,  512)) #0x78
   list.append(Device(121, "ST2000DM001-9YN164", "Barracuda 7200.14 (AF)", 6, 2000,  7200, 4096)) #0x79
   list.append(Device(122, "ST2000DM001-1CH164", "Barracuda 7200.14 (AF)", 6, 2000,  7200, 4096, False, True)) #0x7A
   list.append(Device(123, "ST2000NM0033-9ZM175", "Enterprise Constellation ES3", 6, 2000,  7200, 512, True)) #0x7B
   list.append(Device(124, "TOSHIBA_MG03ACA200", "Toshiba Enterprise", 6, 2000,  7200, 512, True)) #0x7C
   list.append(Device(125, "ST2000DM001-1E6164", "Barracuda 7200.14 (AF)", 6, 2000,  7200, 4096, False, True)) #0x7D
   list.append(Device(126, "ST2000DM001-1ER164", "Barracuda 7200.14 (AF)", 6, 2000,  7200, 4096, False, True)) #0x7D
   list.append(Device(127, "ST2000NM0011",       "Enterprise Constellation ES", 6, 2000,  7200, 512, True)) #0x7B
   list.append(Device(128, "ST2000DM006-2DM164", "Barracuda 7200.14 (AF)", 6, 2000,  7200, 4096, False, True)) #0x7D

   
   list.append(Device(130, "HDS723030ALA640",    "Hitachi",                3, 3000,  7200,  512)) #0x82
   list.append(Device(131, "ST33000651AS",       "Barracuda XT",           6, 3000,  7200,  512)) #0x83
   list.append(Device(132, "ST3000DM001-9YN166", "Barracuda 7200.14 (AF)", 6, 3000,  7200, 4096)) #0x84
   list.append(Device(133, "ST3000DM001-1CH166", "Barracuda 7200.14 (AF)", 6, 3000,  7200, 4096, False, True))  #0x85
   #list.append(Device(134, "WD30EFRX-68AX9N0",   "WDC RED", 6, 3000,  5400, 4096))
   #list.append(Device(135, "ST3000NC002-1DY166", "Barracuda 7200.14 (AF)", 6, 3000,  7200, 4096))
   #list.append(Device(136, "ST3000NC000", "Barracuda 7200.14 (AF)", 6, 3000,  7200, 4096))
   #list.append(Device(137, "WD3000F9YZ-09N20L0", "WDC SE (Cloud)", 6, 3000,  7200, 4096))
   #list.append(Device(138, "WD3001FAEX-00MJRA0", "WDC BLACK", 6, 3000,  7200, 512))
   list.append(Device(139, "ST3000NM0033-9ZM178", "Enterprise Constellation ES3", 6, 3000,  7200, 512, True)) #0x8B
   
   list.append(Device(140, "ST4000DM000-1F2168",  "Barracuda 7200.14 (AF)", 6, 4000,  5900, 4096)) #0x8C
   list.append(Device(141, "ST4000NM0033-9ZM170", "Enterprise Constellation ES3", 6, 4000,  7200, 512, True)) #0x8D
   list.append(Device(142, "WDC_WD4001FAEX-00MJRA0", "WDC BLACK", 6, 4000,  7200, 512))  #0x8E
   list.append(Device(143, "WDC_WD4003FZEX-00Z4SA0", "WDC BLACK", 6, 4000,  7200, 4096)) #0x8F
   list.append(Device(144, "TOSHIBA_MG03ACA400", "Toshiba Enterprise", 6, 4000,  7200, 512, True)) #0x90
   list.append(Device(145, "WDC_WD4004FZWX-00GBGB0", "WDC BLACK", 6, 4000,  7200, 4096)) #0x91
   list.append(Device(146, "ST4000VN008-2DR166", "Seagate IronWolf 4TB NAS", 6, 4000,  7200, 4096, False, True))  #0x92
   
   list.append(Device(147, "ST10000NM0086-2AA101", "Seagate Enterprise 10tb",6, 10000, 7200, 4096))
   list.append(Device(148, "ST12000NM0007-2A1101", "Seagate Enterprise 12tb",6, 12000, 7200, 4096))

   list.append(Device(150, "ST3000DM001-1ER166", "Barracuda 7200.14 (AF)", 6, 3000,  7200, 4096, False, True)) #0x96
   list.append(Device(151, "TOSHIBA_MG03ACA300", "Toshiba Enterprise", 6, 3000,  7200, 512, True)) 
   list.append(Device(152, "ST3000DM001-1E6166", "Barracuda 7200.14 (AF)", 6, 3000,  7200, 4096, False, True))
   list.append(Device(153, "TOSHIBA_MG04ACA300A", "Toshiba Enterprise", 6, 3000,  7200, 4096, True)) 
   list.append(Device(154, "TOSHIBA_MG04ACA400N", "Toshiba Enterprise", 6, 4000,  7200, 512, True))
   list.append(Device(155, "ST3000DM008-2DM166", "Barracuda 7200.14 (AF)", 6, 3000,  7200, 4096, False, True))
   list.append(Device(156, "ST2000NM0055-1V4104", "Enterprise Constellation", 6, 2000, 7200, 512, True))
   list.append(Device(157, "TOSHIBA_MC04ACA400E", "Toshiba Enterprise Cloud", 6, 4000, 7200, 512, True))
   list.append(Device(158, "ST6000VN0041-2EL11C", "Seagate Ironwolf NAS", 6, 6000, 7200, 512, True))
   list.append(Device(159, "ST8000VN0022-2EL112", "Seagate Ironwolf NAS", 6, 8000, 7200, 512, True))

   list.append(Device(160, "ST6000NM0024-1HT17Z", "Enterprise Constellation ES3", 6, 6000,  7200, 4096, True))
   list.append(Device(161, "HGST_HUH728080ALE601", "HGST Ultrastar He8",          6, 8000,  7200, 4096, True))
   list.append(Device(162, "ST8000NM0055-1RM112", "Enterprise Constellation",     6, 8000,  7200, 4096, True))
   list.append(Device(163, "ST6000AS0002-1N917X", "Archive HDD",                  6, 6000,  5900, 4096, True))
   list.append(Device(164, "ST6000NM0115-1YZ110", "Enterprise Constellation v5",  6, 6000,  7200, 4096, True))
   list.append(Device(165, "SuperMicro_SSD",      "Super Micro DOM",              6, 128,  99999,  512))

   list.append(Device(200, "ST2000NM0034", "SEAGATE SAS", 12, 2000,  7200, 512, True))
   list.append(Device(201, "ST2000NM0045", "SEAGATE SAS", 12, 2000,  7200, 512, True))

   list.append(Device(255, "N/A", "N/A", 0, 0,  0, 0, False)) 

   if doPrint:
      for aDevice in list:
         print aDevice
            
   return list     
