import os,sys   
import ConfigParser
abs_path = os.path.abspath(sys.argv[0])
basename = os.path.basename(sys.argv[0])
dirname = abs_path.replace(basename, "")
if not "%s/diskmon/lib" % dirname in sys.path:
   sys.path.insert(0,"%s/diskmon/lib" % dirname) # always with regard to parent script
if not "/usr/bp/lib/monitoring" in sys.path:
   sys.path.append("/usr/bp/lib/monitoring")
from disk      import *   # include config << bpdb as well 
from support_message      import *   # include config << bpdb as well 

#raidInfo code:
#Ex=-1: verified healthy (not done)
#Ex=0: healthy
#Ex=1: rebuild in progress
#Ex=2: verify operation in progress
#Ex=3: verify  is initiated
#Ex=4: degraded
#Ex=5: replace the drive
# (routine evalution of hard disk subsystems in progress)


def get_state_index(state):
   states = ['OPTIMAL', 'PARTIALLY DEGRADED', 'DEGRADED', 'OFFLINE']
   for i,s in enumerate(states):
      if s == state:
         return i

def find_word_wrt_key(key, link, splitter="", offset=0):
   if len(splitter)==0:
      words = link.split()
   else: 
      words = link.split(splitter)
   idx, theIdx =0, 0
   for word in words:
      if key in word:
         theIdx = idx
      idx = idx + 1

   return words[theIdx+offset]

def doRaidInfo(pendingDisks, problemDisks):

   allowedModels=[
      '710','711','712','713','812','813','822','823','824','713S','714S','814S','823S','824S',
      '943','933S','936S','943S','944S','946S',
      '818S','926S','928S','938S','8004','8006','8008','8010','8012','8024S','8032S','8040S',
      '8060S','8080S','8100S','8120S'
      ]
   
   
   #if len(self.pendingDisks) == len(self.problemDisks) or CONFIG.testOn == "raidInfo":  # raidInfo repair
   if CONFIG.modelNo not in allowedModels: return False

   # For future use...
   #excludedModels=['2U','3U','620']
   #if CONFIG.modelNo in excludedModels: return False

   if not CONFIG.selfhealing: return False
   if not CONFIG.confirm:     return False # FILTER IS NEEDED TO AVOID OVER AND OVER TRY
   if not CONFIG.Enable_raidInfo: return False  # default is True
   
   if CONFIG.testOn == "raidInfo":
      if CONFIG.verbose == "raidInfo":
         print CONFIG.modelNo, allowedModels
      return True
   
   if not (len(pendingDisks)): return False
   if not (len(problemDisks)): return False
   if len(pendingDisks) != len(problemDisks): return False
   
   return True

#################class RAID#########################
class RAID:
   def __init__(self, name, ndisk, status, depth=-1):
      self.name  = name
      self.ndiskPerSpan = ndisk

      self.depth =1
      if depth >1: self.depth=depth
      
      self.ndisk = ndisk*depth
      self.status = status        #Optimal, Partically Degraded, Degraded
      self.nmiss = 0
      self.disks,self.degradedDisks,self.reallocDisks,self.serDisks =[],[],[],[]
      self.deadDisks,self.pendingDisks=[],[]
      self.problemDisks =[] # either reallocDisk, serDisk, pendingDisk

      self.message = name + " ++++ %s" % status

      if status!="Optimal" and status !="OK":
          status = red(status)

      self.printMessage = name + " ++++ %s" % status
      self.progressMessage =""

      self.diskConfig = ConfigParser.ConfigParser()
      if os.path.exists(CONFIG.configFile):
         self.diskConfig.read(CONFIG.configFile)
      

######################      
class RAID_LSI(RAID):
   def __init__(self, vdname, device, ndisk, size, status, raidLevel, cachePolicy, depth=-1, IsVDCached=False, bgiProgress=""):
      self.VD_Id = int(vdname[-1:])
      self.device = device
      self.IsVDCached=IsVDCached
      self.bgiProgress = bgiProgress
      
      segs=size.split()
      theSize=float(segs[0])
      TB=theSize
      if segs[1]=="GB":         TB /= 1024.
      if segs[1]=="TB":    theSize *= 1024
         
      self.size = theSize   # GB
      self.TB=int(TB+0.5)
      
      self.raidLevel= (raidLevel.split(",")[0])[-1:]
      self.writemode= cachePolicy.split(",")[0].strip()
      if depth==2: self.raidLevel += "0"

      name = vdname
      RAID.__init__(self, name, ndisk, status, depth)  # self.ndisk = ndisk*depth

      self.healthy = not "Degrade" in self.status and not "Offline" in self.status
      words = name.split()
      first = words[0]
      second = words[1]
      third = words[2]

      effNDisk=self.ndisk
      if self.raidLevel == "1":         effNDisk /= 2
      if self.raidLevel == "6":         effNDisk -= 2
      if self.raidLevel == "60":        effNDisk -= 4

      self.diskSize = self.size/effNDisk

      self.shortName = first[0] + second[0] + third #Virtual Drive: 0
      self.progressMessage = ""
      if len(self.bgiProgress):
         self.progressMessage = "(Background Initialization: %s)" % self.bgiProgress
      if CONFIG.testOn == "LSIBGI":
         self.progressMessage = "(Background Initialization: 10%)"
         
         
   def __repr__(self):
      if "Cache" in self.name:
         return "%s (%s) ++++ %s" % (self.name, self.writemode, self.status)
      else:
         log = "RAID %s (RAID-%s: %s) ++++ %s" % (self.name, self.raidLevel, self.writemode, self.status)
         if len(self.progressMessage):
            log += " " + self.progressMessage
         return log   
         
   def alert(self): #LSI
      if "raid" in CONFIG.verbose:
         print "raid.py:: ", self.status, self.shortName, self.ndisk, self.nmiss, self.size, len(self.degradedDisks), len(self.problemDisks)

      if self.healthy:  # RAID is Okay

         rets = get_printout("ls /var/log/monitoring_disks/*SelfHealingAttempted_%s 2>&1| grep -v 'No such'" % self.shortName)
         if len(rets)>0:
            os.system("mv %s/*SelfHealingAttempted_%s %s/" \
                         % (CONFIG.directory_paths[CONFIG.ibase], self.shortName, CONFIG.directory_paths[CONFIG.iretired]))

         if len(self.problemDisks)==0 and not CONFIG.testOn == "raidInfo":
            CloseAnyActiveAlerts(self.shortName) # shortName is usually VD name
            os.system("echo 0 > /var/log/HVfyopt.log")
         else:
            retIdx =9
            if doRaidInfo(self.pendingDisks, self.problemDisks):  # check if condition is satisfied to run raidInfo
               theCommand="%s diskmon" % CONFIG.raidInfo_Path
               if CONFIG.testOn == "raidInfo":
                  theCommand="%s test" % CONFIG.raidInfo_Path
               lines=get_printout(theCommand)
               if CONFIG.testOn == "raidInfo":
                  print theCommand,lines
               for line in lines:
                  if len(line) !=1: continue
                  retIdx =int(line)
                  break
                  
            if retIdx >3:   
               trapTarget  = self.shortName + " SMT"
               trapMessage = self.name + " SMART concerns due to disk(s):"
               for disk in self.problemDisks:
                  trapTarget  += disk.trapTargAdd 
                  trapMessage += disk.trapMessAdd + "(**Needs Review**: %s)" % disk.smart_for_trap
               if CONFIG.testOn == "raidInfo":
                  trapTarget  += "Test"
                  trapMessage += "(**Needs Review**)"
                  
               trapMessage += " >>%d" % retIdx   
               if CONFIG.selfhealing: # delayedAlert handles... and CONFIG.dbON:
                  notification_id = get_notification_id(trapMessage)                  
                  HandleAlert(self.shortName, notification_id, trapTarget, trapMessage, "NoTrapWeekend") # shortName is usually VD name
               
      else: #if "Degrade" in self.status or "DEGRADE" in self.status:
         trapTarget  = self.shortName
         trapMessage = self.message + " due to disk(s):"

         if len(self.degradedDisks):  # MEANING: DEGRADE and Drive is recognized
            for disk in self.degradedDisks:
               trapTarget  += disk.trapTargAdd + "%s" % disk.statusInitial 
               trapMessage += disk.trapMessAdd
               trapMessage = disk.SelfHeal(trapMessage) ##############
         else:                        # MEANING: DEGRADE and Drive is dead
            lines=[]
            if self.nmiss:
               lines =get_printout("ls -t /var/log/monitoring_disks/old/*dat 2>&1 | grep -v cannot")
            for i in range(len(lines)):
               if self.nmiss == i: break
               line=lines[i]
               words=line.split(".")
               segs = words[0].split("/")
               theModel = segs[len(segs)-1]
               theSN = words[1]
               trapTarget += " B99DD"
               trapMessage += " [Bay99](**Needs Review** Model: %s SN: %s DD)" % (theModel,theSN)

         if "raid" in CONFIG.verbose:
            print "raid.py::  %s" % trapMessage, "<<<", len(self.degradedDisks)
               
         if len(self.problemDisks)!=0:
            trapTarget  += " SMT"
            trapMessage += " ++++ SMART concerns due to disk(s):"
            for disk in self.problemDisks:
               trapTarget  += disk.trapTargAdd
               trapMessage += disk.trapMessAdd + "(**Needs Review**: %s)" % disk.smart_for_trap

         if CONFIG.selfhealing: # delayedAlert handles... and CONFIG.dbON:
            notification_id = get_notification_id(trapMessage)
            HandleAlert(self.shortName, notification_id, trapTarget, trapMessage) # shortName is usually VD name

         if "raid" in CONFIG.verbose: print "raid.py:: ", len(self.degradedDisks), len(self.problemDisks), trapMessage, trapTarget, "<<end"

      ###
      if len(self.problemDisks)!=0:
         if len(self.pendingDisks)==len(self.problemDisks):
            showMessage("LSI", "SMTP")
         else:
            showMessage("LSI", "SMT")

      return self.nmiss or len(self.degradedDisks)

###################
class RAID_3ware(RAID):
   def __init__(self, name, ndisk, status, depth=-1, nNotOpt =-1, raidlevel="", subunits=[], ignoreECC_status='', unit_name=''):
      RAID.__init__(self, name, ndisk, status, depth)
      self.nmiss = nNotOpt
      self.shortName = name
      self.subunits = subunits
      self.raidLevel = raidlevel
      self.tw_cli_output = get_printout("%s info %s" % (CONFIG.tw_cliPath, name))
      self.tw_cli_unitstatus=""
      for line in self.tw_cli_output:
         if not "RAID" in line: continue
         self.tw_cli_unitstatus =line
         
      self.writemode  = self.tw_cli_unitstatus.split()[7]
      self.autoverify = self.tw_cli_unitstatus.split()[8]
      self.printMessage =""
      self.ignoreECC_status = ignoreECC_status
         
      nOKDrives,nDGDrives =0,0
      for ue in subunits: # this is for port list

         if CONFIG.verbose=="raid":
            print ue.unitIndex, ue.vport, ue.status
            
         if ue.status == "NOT-PRESENT":
            continue 
         if ue.status == "OK":
            nOKDrives += 1
         else:  # if ue.status == "DEGRADED": there is "-"
            nDGDrives += 1

      self.nOKDrives = nOKDrives
      self.nDGDrives = nDGDrives

      if CONFIG.verbose=="raid":
         print self.nOKDrives, self.nDGDrives
      
      if status == "REBUILDING" or status == "REBUILD-VERIFY" or "RBLD" in status or "REBUILD-INIT" in status:  #DEGRADED-RBLD
         self.progressMessage = self.tw_cli_unitstatus.split()[3]
         self.printMessage = self.message + " " + self.progressMessage
      elif "INIT" in status or status == "VERIFYING":  # INITIALIZING or DEGRADED-INIT
         self.progressMessage = self.tw_cli_unitstatus.split()[4]
         self.printMessage = self.message + " " + self.progressMessage

      # 3ware is different from LSI in terms of status
      # LSI: while rebuilding still Degrade status
      # 3ware: while rebuilding it becomes rebuild
      self.healthy =not "DEGRADED" in self.status 
      self.rebuild = self.status in ("REBUILDING","REBUILD-VERIFY","INITIALIZING")
      
      if self.status in ("REBUILDING", "DEGRADED-RBLD", "REBUILD-VERIFY"):
         if self.ignoreECC_status != "on":
            run_command("%s /%s/%s set ignoreECC=on"%(CONFIG.tw_cliPath,name,unit_name),0,True) # 0s sleep and confirm
      else:
         if self.ignoreECC_status != "off":
            run_command("%s /%s/%s set ignoreECC=off"%(CONFIG.tw_cliPath,name,unit_name),0,True) # 0s sleep and confirm


            
   def __repr__(self):
      return "RAID %s (%s: %s,%s) ++++ %s %s" \
          % (self.name, self.raidLevel, self.writemode, self.autoverify, self.status, self.printMessage)


   def alert(self): # 3ware
      if self.healthy:
         
         if len(self.problemDisks)==0 and not CONFIG.testOn == "raidInfo":
            if not self.rebuild:
               CloseAnyActiveAlerts(self.shortName)
               #WP Delete All the Files...
               rets = get_printout("ls /var/log/monitoring_disks/*SelfHealingAttempted 2>&1| grep -v 'No such'")
               if len(rets)>0:
                  os.system("mv %s/*SelfHealingAttempted %s/" \
                               % (CONFIG.directory_paths[CONFIG.ibase], CONFIG.directory_paths[CONFIG.iretired]))
               os.system("echo 0 > /var/log/HVfyopt.log")
            else:  # when rebuild is running
               trapTarget  = self.shortName
               trapMessage = self.message
               for disk in self.degradedDisks:
                  if disk.statusInitial=="R":
                     trapMessage += "(Rebuild Progress %s: %s)" % (self.progressMessage, disk.smart_for_trap)
               notification_id = 138
               HandleAlert(self.shortName, notification_id, trapTarget, trapMessage) # shortName is usually VD name
               
         else:
            retIdx =9
            if doRaidInfo(self.pendingDisks, self.problemDisks):
               theCommand="%s diskmon" % CONFIG.raidInfo_Path
               if CONFIG.testOn == "raidInfo":
                  theCommand="%s test" % CONFIG.raidInfo_Path
               lines=get_printout(theCommand)
               if CONFIG.testOn == "raidInfo":
                  print theCommand,lines
               for line in lines:
                  if len(line) !=1: continue
                  retIdx =int(line)
                  break
               
            if retIdx >3:   
               trapTarget = self.shortName + " SMT"
               trapMessage = self.name + " SMART concerns due to disk(s):"
               for disk in self.problemDisks:
                  trapTarget  += disk.trapTargAdd
                  trapMessage += disk.trapMessAdd + "(**Needs Review**: %s)" % disk.smart_for_trap
               if CONFIG.testOn == "raidInfo":
                  trapTarget  += "Test"
                  trapMessage += "(**Needs Review**)"

               trapMessage += " >>%d" % retIdx   
               if CONFIG.selfhealing: #delayedAlert handles... and CONFIG.dbON:
                  notification_id = get_notification_id(trapMessage)
                  # shortName is usually controller name
                  HandleAlert(self.shortName, notification_id, trapTarget, trapMessage, "NoTrapWeekend") 
                  
      else: #if "Degrade" in self.status or "DEGRADE" in self.status:
         trapTarget  = self.shortName
         trapMessage = self.message + " due to disk(s):"
                                    
         nPendingDisks = len(self.pendingDisks)
         nDegradedDisks = len(self.degradedDisks)

         if CONFIG.verbose=="raid":
            print len(self.degradedDisks)

         for disk in self.degradedDisks:
            trapTarget  += disk.trapTargAdd + "%s" % disk.statusInitial 
            trapMessage += disk.trapMessAdd
            trapMessage = disk.SelfHeal(trapMessage, self.raidLevel, nPendingDisks, nDegradedDisks, self.progressMessage) ##############

         if len(self.problemDisks)!=0:
            trapTarget  += " SMT"
            trapMessage += " ++++ SMART concerns due to disk(s):"
            for disk in self.problemDisks:
               trapTarget  += disk.trapTargAdd
               trapMessage += disk.trapMessAdd + "(**Needs Review**: %s)" % disk.smart_for_trap
                           
         if CONFIG.selfhealing: # delayedAlert handles.... and CONFIG.dbON
            notification_id = get_notification_id(trapMessage)
            HandleAlert(self.shortName, notification_id, trapTarget, trapMessage) # shortName is usually controller name

      if len(self.problemDisks)!=0:
          showMessage("3ware", "SMT")
          
      return len(self.degradedDisks)

def get_mdraids():
   #for all systems mirrored raids self healing is broken, this will attempt to correct them during discovery only and will attempt a one time fix
   import commands
   lines = commands.getoutput("sudo -iu root /usr/bp/bin/disk_mon -mdr")
   devices = [] # this is from /proc/mdstat
   mdraids = []
   status = "OK"
   deg_statcnt =0

   command="grep -v Personalities /proc/mdstat"
   if CONFIG.testOn == "SWRAID":
      command="cat /root/benchmark/swraid.out"
   for line in get_aggregated_lines(command):
      if not "U" in line: continue
      estimate, substatusBit, progress, substatus = "", "", "Done", "OK" 
      subdevices, subdeviceStatus =[], []
      words = line.split()
      for word in words:
         if "U" in word:
            substatusBit = word[1:len(word)-1]
            if "recovery" in line:   # re-add / add: you will see recovery  
               substatus = "recovery"
            elif "resync" in line:   # mdadm v3.2.6  recovery status considered as U and called resync
               substatus = "resync"  # resync=DELAYED
         if "_" in word:
            if "recovery" in line:
               substatus = "recovery"
            elif "resync" in line:
               substatus = "resync"  # resync=DELAYED
            elif "(F)" in line or "(S)" in line:
               progress = "NotInitiated"
               deg_statcnt += 1
               substatus = "failed"
               if "(F)" in line and "(S)" in line:
                  substatus = "failed_and_stopped"
               elif "(S)" in line:
                  substatus = "stopped"

            else:   
               substatus = "Degraded"
               progress = "NotInitiated"
               deg_statcnt += 1
         elif "md" in word:
            raidName = word
         elif "%" in word:
            progress = word
         elif "DELAYED" in word:
            progress = "DELAYED"
         elif "finish=" in word:
            estimate = word
         elif "sd" in word:
            device = ""
            if "(F)" in word:
               subdevices.append(word[0:len(word)-6])
               subdeviceStatus.append("F")
               device = word[0:len(word)-7]
            elif "(S)" in word:
               subdevices.append(word[0:len(word)-6])
               subdeviceStatus.append("S")
               device = word[0:len(word)-7]
            else:
               subdevices.append(word[0:len(word)-3])
               subdeviceStatus.append("O")
               device = word[0:len(word)-4]
            if not device in devices:
               devices.append(device)
      mdraids.append(MdRaid(raidName, substatus, substatusBit, progress, subdevices, subdeviceStatus, estimate))

   if deg_statcnt > 0:
       status = "Degraded"
   return (status, devices, mdraids)    


########
class RAID_OS(RAID):
   def __init__(self, appliance): #, config):
      #self.config = config
      ###
      self.appliance=appliance
      (status, devices, mdraids) =get_mdraids()   
      mdraids.sort(key=lambda MdRaid: MdRaid.name)

      self.N_DegMdRaid, self.N_RebuildMd = 0,0
      for mdraid in mdraids:
         if "Degraded" == mdraid.status: self.N_DegMdRaid+=1
         if "resync"   == mdraid.status: self.N_RebuildMd+=1
         if "recovery" == mdraid.status: self.N_RebuildMd+=1
      
      depth = 1
      if "7" in appliance.name:
         depth = 2

      ndisk = appliance.nDisk_OS_and_Cache
      if CONFIG.verbose=="RAID_OS":
         print "WP@raid.py: ndisk ", appliance.nDisk_OS_and_Cache

      if "0" == get_first_word("grep -v Personalities /proc/mdstat |grep -v unused| /usr/bin/wc -l") and appliance.controller == "":  # single drive appliance such as 201  
         ndisk = 1
         for line in get_printout("mount | grep dev | grep sd | grep boot"):
            devices.append(line.split()[0][5:8])
            
      if CONFIG.verbose=="RAID_OS":
         print "WP@raid.py: ndisk appliance.mdlevel appliance.name ", ndisk, appliance.mdlevel, appliance.name
         print "WP@raid.py: CentOS_version ", CONFIG.CentOS_version
         for raid in mdraids:
            print "mdraids ;;;", raid
         print "firstHostN_AHCI",CONFIG.firstHostN_AHCI
         print "total_Ndisk", len(appliance.oshosts), "flashchach? ", appliance.flashcache
         
      firstHostN_OS=CONFIG.firstHostN_AHCI
      total_Ndisk =len(appliance.oshosts)
      if appliance.flashcache: total_Ndisk += 1

      unicom_1U,idHosts=False,[]
      if "814S-7" in CONFIG.assetTag or "818S-7" in CONFIG.assetTag:
         unicom_1U=True
         idHosts=[3,5,6,7,8]

      self.ssdHosts, self.liveHosts, liveDevices, md_LiveDevices, md_device_candidates =[],[],[],[],[]

      for scsi in CONFIG.scsi_devices:
         if not ("ATA" in scsi.vendor or "WDC WD" in scsi.vendor or ("ST" in scsi.vendor) or "JMicron" in scsi.vendor): continue
         # REJECT SSD   
         if "0" == get_first_word("cat /sys/block/%s/queue/rotational" % scsi.deviceShort):
            self.ssdHosts.append("host%d" % scsi.hostNumber)
            continue  
         if len(scsi.deviceShort):          liveDevices.append(scsi.deviceShort)
         if unicom_1U:
            if scsi.hostNumber not in idHosts: continue
         else:
            if not (scsi.hostNumber >= firstHostN_OS and scsi.hostNumber <firstHostN_OS+total_Ndisk): continue

         md_device_candidates.append(scsi.deviceShort)
         command="%s -i %s | /bin/grep Model | /usr/bin/wc -l" % (CONFIG.smartctl_Path, scsi.device)
         if len(scsi.deviceShort) and int(get_first_word(command)):
            if scsi.hostNumber >=0:            self.liveHosts.append(scsi.hostNumber)
            md_LiveDevices.append(scsi.deviceShort)
            
      self.missingHosts = []
      if firstHostN_OS >=0:
         if unicom_1U:
            for idHost in idHosts:
               if idHost in self.liveHosts: continue
               theHost ="host%d"%idHost
               if theHost in self.ssdHosts: continue
               self.missingHosts.append(theHost)
         else:
            for idHost in range(firstHostN_OS,firstHostN_OS+total_Ndisk):
               if idHost in self.liveHosts: continue
               theHost ="host%d"%idHost
               if theHost in self.ssdHosts: continue
               self.missingHosts.append(theHost)

      # bug 20324 case: host numbers are host6/7
      self.mdraids = mdraids
      self.disks, self.shortName = [], "OS"
      self.md_devices=devices # this is from mdstat
      self.md_device_candidates=md_device_candidates
      self.devices = md_LiveDevices  # live devices not necessarily part of md array including flashcache in new gen


      RAID.__init__(self, "OS", ndisk, status, depth)
      self.ndisk = ndisk
      self.nmiss = ndisk - len(self.devices) -len(self.ssdHosts)  # devices are from /proc/mdstat so that nmiss and len(self.missingHosts) become inconsistent
      self.message = "OS RAID ++++"
      self.healthy = not "Degraded" in self.status  # set in RAID.__init__
      #self.rebuild = self.status in ("Rebuild")  # redudant flag in addition to self.N_DegMdRaid, self.N_RebuildMd

      """
      ndisk: # of disk for mdraid
      nmiss: # of dead disk
      md_devices: device names currently in mdraid
      md_device_candidates: device name supposed to be in mdraid
      liveHosts: host numbers which are not dead
      liveDevices: any live devices including archiving devices
      md_LiveDevices: just like md_device_candidates
      missingHosts: dead drive host number
      ssdHosts: host number for ssd
      """


      if CONFIG.verbose=="RAID_OS":
         print "WP@raid.py: ndisk ", ndisk
         print "WP@raid.py: nmiss", self.nmiss, len(self.devices), len(self.ssdHosts)
         for scsi_device in CONFIG.scsi_devices:
            print "WP@raid.py: CONFIG.scsi_devices", scsi_device
         print "WP@raid.py: md_devices ", devices 
         print "WP@raid.py: md_device_candidates ", md_device_candidates
         print "WP@raid.py: liveHosts ", self.liveHosts 
         print "WP@raid.py: liveDevices (self.devices)", liveDevices 
         print "WP@raid.py: md_LiveDevices (self.devices)", md_LiveDevices 
         print "WP@raid.py: missingHosts ", self.missingHosts
         print "WP@raid.py: ssdHosts ", self.ssdHosts

   def __repr__(self):
      log =""
      for i  in range(len(self.mdraids)):
         mdraid=self.mdraids[i]
         log += "%s" % mdraid
         if i+1<len(self.mdraids):
            log +="\n"
      return log   
         
   def alert(self): ### RAID_OS
      import commands

      # excluding archive (used to be in consideration)
      theProblemDisks_rebuild=[]
      #thePendingDisks_rebuild, theReallocDisks_rebuild, theSerDisks_rebuild=[],[],[]
      theProblemDisks, thePendingDisks, theReallocDisks, theSerDisks=[],[],[],[]
      for disk in self.problemDisks:
         theProblemDisks.append(disk)
         #if not disk.archive:
         #   theProblemDisks_rebuild.append(disk)

      # The following is for rebuild to make sure to zero in pending sector for any drive
      for disk in self.disks:
         if disk.smart.pending or disk.smart.uncorr:
            if disk not in theProblemDisks_rebuild:
               theProblemDisks_rebuild.append(disk)
      #######
               
      for disk in self.pendingDisks:
         thePendingDisks.append(disk)
         #if not disk.archive:
         #   thePendingDisks_rebuild.append(disk)
      for disk in self.reallocDisks:
         theReallocDisks.append(disk)
         #if not disk.archive:
         #   theReallocDisks_rebuild.append(disk)
      for disk in self.serDisks:
         theSerDisks.append(disk)
         #if not disk.archive:
         #   theSerDisks_rebuild.append(disk)

      if self.healthy:
         if len(theProblemDisks)==0 and not CONFIG.testOn == "raidInfo":
            if not self.N_RebuildMd:
               CloseAnyActiveAlerts(self.shortName)
               #WP Delete All the Files...
               rets = get_printout("ls /var/log/monitoring_disks/*SelfHealingAttempted_md* 2>&1| grep -v 'No such'")
               if len(rets)>0:
                  os.system("mv %s/*SelfHealingAttempted_md* %s/" % (CONFIG.directory_paths[CONFIG.ibase], CONFIG.directory_paths[CONFIG.iretired]))
               os.system("echo 0 > /var/log/SVfyopt.log")

            else:  # when rebuild is running
               trapTarget  = self.shortName 
               trapMessage = self.message + " (Rebuild Progress)"
               notification_id = 138
               HandleAlert(self.shortName, notification_id, trapTarget, trapMessage) # shortName is usually VD name

         else:  # Only SMART ISSUE
            retIdx =-9
            if doRaidInfo(thePendingDisks, theProblemDisks):
               theCommand="%s diskmon" % CONFIG.raidInfo_Path
               if CONFIG.testOn == "raidInfo":
                  theCommand="%s test" % CONFIG.raidInfo_Path
               lines=get_printout(theCommand)
               if CONFIG.testOn == "raidInfo":
                  print theCommand,lines
               for line in lines:
                  if len(line) !=1: continue
                  retIdx =int(line)
                  break
                  
            if retIdx >3 or "sd" in CONFIG.testOn:   
               trapTarget = self.shortName + " SMT"
               trapMessage = self.name + " SMART concerns due to disk(s):"

               is65K_PND=False
               for disk in theProblemDisks:
                  if disk.smart.pending>65000: is65K_PND=True
                  trapTarget  += disk.trapTargAdd
                  revisedSMART = disk.smart_for_trap
                  if disk.archive:
                     revisedSMART= "Archv %s" % revisedSMART
                  trapMessage += disk.trapMessAdd + "(**Needs Review**: %s)" % revisedSMART
                  
               if CONFIG.testOn == "raidInfo":
                  trapTarget  += "Test"
                  trapMessage += "(**Needs Review**)"

               trapMessage += " >>%d" % retIdx   
               # Need to lower severity for 65K pending sector:  3/24/16
               if len(theProblemDisks)==1 and len(thePendingDisks)==1 and is65K_PND:
                  trapMessage +=" <<benignCase"
                  
               if CONFIG.selfhealing: #delayedAlert handles... and CONFIG.dbON:
                  notification_id = get_notification_id(trapMessage)
                  HandleAlert(self.shortName, notification_id, trapTarget, trapMessage, "NoTrapWeekend") # shortName is usually OS name
               
      else: #if "Degrade" in self.status or "DEGRADE" in self.status:
         trapTarget  = self.shortName
         trapMessage = self.message + " Degraded"
         if CONFIG.verbose=="RAID_OS":
            print "WP: ", trapMessage, self.nmiss

         if self.nmiss > 0: # in case there is missing drive!!!! most likely dead drive
            trapMessage += " due to offline disk"
            if CONFIG.verbose=="RAID_OS":
               print len(self.missingHosts), self.missingHosts
            for i in range(len(self.missingHosts)):
               host = self.missingHosts[i]
               bayNumber = int(host[4:])-CONFIG.firstHostN_AHCI
               if self.appliance.flashcache_in_front:
                  bayNumber -=1
               
               trapTarget += " BAY%d" % bayNumber
               theModel,theSN="",""
               if self.diskConfig.has_section(host):
                  theModel=self.diskConfig.get(host, "Model")
                  theSN=self.diskConfig.get(host, 'Serial')
               if CONFIG.verbose=="RAID_OS":
                  print "offline disk ",theModel, " - ", theSN
               if "NotAvailable" in theModel or not len(theModel):
                  ref_models=[]
                  for ihost in self.liveHosts:
                     ref_host="host%d"%ihost
                     ref_model=""
                     if self.diskConfig.has_section(ref_host):
                        ref_model=self.diskConfig.get(ref_host, "Model")
                     if len(ref_model) and ref_model not in ref_models:
                        ref_models.append(ref_model)

                  grep_string=""
                  for ref_model in ref_models:
                     grep_string+="%s|" %ref_model
                  if len(grep_string): grep_string=grep_string[:-1]
                        
                  ls_command = "ls -t %s/*.dat" % (CONFIG.directory_paths[2])
                  if len(grep_string):
                     ls_command += "| grep -E '%s'" % grep_string
                  if CONFIG.verbose=="RAID_OS":
                     print i, len(self.missingHosts), self.nmiss, ls_command
                     print ls_command
                  diskFile = get_printout(ls_command)[i]
                  words=diskFile.split("/")
                  segs = words[len(words)-1].split(".")
                  theModel=segs[0]
                  theSN=segs[1]
               trapMessage += " BAY%d" %bayNumber + "(Model:%s" % theModel + " Serial:%s)" % theSN
               if CONFIG.verbose=="RAID_OS":
                  print "WP: ", trapMessage, self.nmiss
              
            trapMessage += " (**Needs Review** DD)" 
            if not CONFIG.selfhealing:
               log = "[ALERT] Degraded due to the offline disk"
               for i in range(len(self.missingHosts)):
                  host = self.missingHosts[i]
                  theModel,theSN="",""
                  if self.diskConfig.has_section(host):
                     theModel=self.diskConfig.get(host, "Model")
                     theSN=self.diskConfig.get(host, 'Serial')
                  if "NotAvailable" in theModel or not len(theModel):
                     ref_models=[]
                     for ihost in self.liveHosts:
                        ref_host="host%d"%ihost
                        ref_model=""
                        if self.diskConfig.has_section(ref_host):
                           ref_model=self.diskConfig.get(ref_host, "Model")
                        if len(ref_model) and ref_model not in ref_models:
                           ref_models.append(ref_model)
   
                     grep_string=""
                     for ref_model in ref_models:
                        grep_string+="%s|" %ref_model
                     if len(grep_string): grep_string=grep_string[:-1]
                           
                     ls_command = "ls -t %s/*.dat" % (CONFIG.directory_paths[2])
                     if len(grep_string):
                        ls_command += "| grep -E '%s'" % grep_string
                     diskFile = get_printout(ls_command)[i]
                     words=diskFile.split("/")
                     segs = words[len(words)-1].split(".")
                     theModel=segs[0]
                     theSN=segs[1]
                  log = log + " " + host + " (Model: %s" % theModel + " Serial: %s)" % theSN
               print log    
               print "        ==> Please, initiate warranty request ONLY IF NEEDED"
               print "        ==> Useful commands : ls -l /sys/block/sd*/device"
         else:  #   # doable selfhealing candidate
            #diskProblem = len(theReallocDisks) or len(theSerDisks)  # see if disk drive has an issue

            nDegraded, nFailed, nAttempted = 0, 0, 0
            preperIdx=-1

            #####
            # Prep for partitioning new drive
            #########
            if CONFIG.selfhealing and CONFIG.confirm:
               preper=SWRAID_Preparator(self)
               if CONFIG.verbose=="RAID_OS_SelfHeal":
                  print "WP1: ", preper.exitId

               if not preper.exitId:

                  command="echo '=== %s (%d)' >> %s" % (CONFIG.now, preper.exitId, CONFIG.swrebuild_logfile)
                  os.system(command)
                  preperIdx=preper.write_table()
                  command="echo '    partner: %s to_add: %s (%d)' >> %s" \
                      % (preper.partner_device, preper.device_to_replace, preperIdx, CONFIG.swrebuild_logfile)
                  os.system(command)

                  # In case of new drive detected... ignore SelfHealingAttempted....
                  command="mv %s/*SelfHealingAttempted_md* %s/" % (CONFIG.directory_paths[CONFIG.ibase], CONFIG.directory_paths[CONFIG.iretired])
                  os.system("echo '    %s'  >> %s" % (command, CONFIG.swrebuild_logfile))
                  os.system(command)
               
                  if CONFIG.verbose=="RAID_OS_SelfHeal":
                     print "WP2: preper.write_table() is run ", preperIdx
            

            for mdraid in self.mdraids:
               if mdraid.status == "Degraded":   nDegraded += 1
               if mdraid.status == "failed"  :   nFailed   += 1
               #if diskProblem: continue  # with disk problem (realloc or ser) we don't want to proceed self-healing
               #if not CONFIG.dev and self.ndisk != 2: continue  # only for RAID 1 with 2 drives 
               if not CONFIG.selfhealing or not CONFIG.confirm: continue # FILTER IS NEEDED TO AVOID OVER AND OVER TRY
               ##################
               # SelfHealing part   
               ##################
               if os.path.exists("%s/SelfHealingAttempted_%s" % (CONFIG.directory_paths[CONFIG.ibase], mdraid.name)):
                   continue
               Attempted = False

               if CONFIG.verbose=="RAID_OS_SelfHeal":
                  print "WP3: ", mdraid.deviceStatus, mdraid.deviceStatus.count('F'),mdraid.statusBit

               #orgNdrive=len(mdraid.statusBit)
               if mdraid.status == "Degraded":
                   # if n devices found in array equals original n devices
                   # ['O', 'O', 'O'] compared to _UUU in case of degraded....
                   #print self.devices, mdraid.devices, mdraid.deviceStatus
                   #['sdb', 'sdc'] ['sdb2'] ['O']

                   ## TODO:
                   ## SPEED UP PARAMETERS AND NEED TO IMPLEMENT IF TASKER IS TURN OFF
                   ## This may not be viable in self healing production system

                   if len(mdraid.deviceStatus) == len(mdraid.statusBit) : continue  # This is for raid1
                   anyDeviceInRaid = mdraid.devices[0] # sdb2
                   subPartId = anyDeviceInRaid.replace(anyDeviceInRaid[:-1],"") #2

                   deviceToAdd = ""
                   for dev in self.devices: # can't move to mdraid class due to this self.devices!!! :(
                      if "%s%s"%(dev,subPartId) in mdraid.devices: continue
                      if dev in self.md_devices: continue
                      deviceToAdd = dev

                   ## MAKE SURE ALL THE DRIVES in RAID SHOULDN'T HAVE ANY ISSUE
                   nProblemDisk_partner=0
                   for disk in theProblemDisks_rebuild:
                      if deviceToAdd in disk.device: continue
                      nProblemDisk_partner+=1
                   if nProblemDisk_partner: continue

                      
                   if CONFIG.verbose=="RAID_OS_SelfHeal":
                      print "WP4: ", self.devices, self.md_devices, mdraid.devices, anyDeviceInRaid,subPartId,deviceToAdd
                   if not len(deviceToAdd):
                      print "WP5: ", subPartId,deviceToAdd
                      continue
                   
                   if not os.path.exists("/dev/%s%s" %(deviceToAdd, subPartId)):
                      print "WP6: NOT EXIST", "/dev/%s%s" %(deviceToAdd, subPartId), os.path.exists("/dev/%s%s" %(deviceToAdd, subPartId))
                      continue

                   command = "/sbin/mdadm --add /dev/%s /dev/%s%s"  % (mdraid.name, deviceToAdd, subPartId)
                   if CONFIG.verbose=="RAID_OS_SelfHeal":
                      print command
                   run_command(command)
                   Attempted = True
                   nAttempted += 1

               elif mdraid.status == "failed":
                   if len(mdraid.deviceStatus) != len(mdraid.statusBit) : continue  # This is for raid1
                   if mdraid.deviceStatus.count('F')>1                  : continue
                   #print self.devices, mdraid.devices, mdraid.deviceStatus
                   #['sdb', 'sdc'] ['sdb3', 'sdc3'] ['O', 'F']
                   deviceToTreat = ""
                   for i in range(len(mdraid.devices)):
                       if mdraid.deviceStatus[i] == "F":
                           deviceToTreat = mdraid.devices[i]
                   if not len(deviceToTreat): continue
                   if not os.path.exists("/dev/%s%s" %(deviceToAdd, subPartId)):
                      continue
                   command = "/sbin/mdadm --remove /dev/%s /dev/%s"  % (mdraid.name, deviceToTreat)
                   run_command(command)
                   command = "/sbin/mdadm --add /dev/%s /dev/%s"  % (mdraid.name, deviceToTreat)
                   run_command(command)

                   Attempted = True
                   nAttempted += 1
               if Attempted:     
                   run_command("echo > %s/SelfHealingAttempted_%s" % (CONFIG.directory_paths[CONFIG.ibase], mdraid.name))

            ### Summary of Self-healing event
            if nDegraded or nFailed:
               if nAttempted:
                  trapMessage += " mdraid(s) "
                  for mdraid in self.mdraids:
                     if mdraid.status=="Degraded":
                        trapMessage += " %s" % mdraid.name

                  trapMessage += " Rebuild in progress "
                  trapMessage += "Self-healing script initiated array rebuild +%d " % nAttempted
                  if preperIdx>=0:
                     trapTarget += " REPLACEMENT"
                     trapMessage += "(preperIdx %d)" % preperIdx
                  showMessage("OS", "RBLD")
               else:
                  trapMessage += " mdraid(s) "
                  for mdraid in self.mdraids:
                     if mdraid.status=="Degraded":
                        trapMessage += " %s" % mdraid.name
                     if mdraid.status=="failed":
                        trapMessage += " %s(F)" % mdraid.name
                     elif mdraid.status=="stopped":
                        trapMessage += " %s(S)" % mdraid.name
                     elif mdraid.status=="failed_and_stopped":
                        trapMessage += " %s(F_S)" % mdraid.name
                  trapMessage += " Degraded (**Needs Review** PD)" #Partial Degrade

                  #for device in degraded_device:
                  #   for disk in self.disks:
                  #      if not device in disk.device: continue
                  #      trapMessage += disk.trapMessAdd + "(%s: %s)" % (messageString, disk.smart_for_trap)
                  showMessage("OS", "FS")

         if len(theProblemDisks)!=0:
            trapTarget  += " SMT"
            trapMessage += " ++++ SMART concerns due to disk(s):"
            for disk in theProblemDisks:
               trapTarget  += disk.trapTargAdd
               revisedSMART = disk.smart_for_trap
               if disk.archive:
                  revisedSMART= "Archv %s" % revisedSMART
               trapMessage += disk.trapMessAdd + "(**Needs Review**: %s)" % revisedSMART
               
         if CONFIG.testOn=="TRAP" or CONFIG.testOn=="SWRAID":
            print trapMessage
               
         if CONFIG.selfhealing: # dbOFF handle belong to delayedAlert  hanand CONFIG.dbON:
            notification_id = get_notification_id(trapMessage)
            HandleAlert(self.shortName, notification_id, trapTarget, trapMessage) # shortName is usually VD name
      ###      
      if len(theProblemDisks)!=0:
          showMessage("OS", "SMT")
          
      return (self.status == "Degraded")    

def get_notification_id(trapMessage):
   notification_id = 0
   if "Needs Review" in trapMessage: # severity becomes 1
      notification_id = 137
   else:   #severity becomes 3
      notification_id = 138
   return notification_id

class MdRaid: 
   """MdRaid which will be subset of RAID_OS""" 
   def __init__(self, name, status, bit, progress, devices, deviceStatus, estimate): #encId is only for LSI drive
      self.name = name
      self.status = status               # MdRaid overall status resync, recovery, OK, Degraded
      self.statusBit = bit               # _UUU
      self.progress = progress
      self.devices = devices             # devices currently intact
      self.deviceStatus = deviceStatus   #['O', 'O', 'O'] 
      self.estimate = ""
      self.stripe_cache_size =""

      if os.path.exists("/sys/block/%s/md/stripe_cache_size" % self.name):
         self.stripe_cache_size=get_first_word("cat /sys/block/%s/md/stripe_cache_size" %self.name)

      self.getra=get_first_word("/sbin/blockdev --getra /dev/%s" % self.name)
      
      if len(estimate)>9:
 	 est = int(float(estimate[7:-3]))
         import datetime
         now = datetime.datetime.now()
         then = now + datetime.timedelta(minutes = est)
         self.estimate = "by %02d-%02d %02d:%02d" %(then.month, then.day, then.hour, then.minute) 

      hosts=[]
      subwords =[]

      for i in range(len(devices)):
         device = devices[i] 
         host = get_hostnumber(device[0:-1])
         hosts.append(host)
         if "F" in self.deviceStatus[i]:
             subwords.append("%s(F)(%s)" % (device,host))
         elif "S" in self.deviceStatus[i]:
             subwords.append("%s(S)(%s)" % (device,host))
         else:    
             subwords.append("%s(%s)" % (device,host))

      self.hosts = hosts
      self.subwords = subwords 


   def __repr__(self):
      log = "RAID /dev/%s" %self.name + " ++++ %8s" % self.status
      if "re" in self.status:   
         log += "%9s" % (self.progress)
      else:   
         log += "         "

      log += "    current disks:"
      temp = self.subwords
      temp.sort()
      for subword in temp:
         log = log + " " + subword 
      log += " %s" % (self.estimate)

      log += " ra:%s " %(self.getra)
      if len(self.stripe_cache_size):  
         log += " stripe:%s " %(self.stripe_cache_size)

      return log


   
class SWRAID_Preparator:
   def __init__(self,raid_os):
   
      self.exitId=0
      """
      ndisk: # of disk for mdraid
      nmiss: # of dead disk
      md_devices: device names currently in mdraid
      md_device_candidates: device name supposed to be in mdraid
      liveHosts: host numbers which are not dead
      liveDevices: any live devices including archiving devices
      md_LiveDevices: just like md_device_candidates
      missingHosts: dead drive host number
      ssdHosts: host number for ssd

      if CONFIG.verbose=="RAID_OS":
         print "WP@raid.py: ndisk ", ndisk
         print "WP@raid.py: nmiss", self.nmiss, len(self.devices), len(self.ssdHosts)
         print "WP@raid.py: CONFIG.scsi_devices", CONFIG.scsi_devices
         print "WP@raid.py: md_devices ", devices 
         print "WP@raid.py: md_device_candidates ", md_device_candidates
         print "WP@raid.py: liveHosts ", liveHosts 
         print "WP@raid.py: liveDevices (self.devices)", liveDevices 
         print "WP@raid.py: md_LiveDevices (self.devices)", md_LiveDevices 
         print "WP@raid.py: missingHosts ", self.missingHosts
         print "WP@raid.py: ssdHosts ", self.ssdHosts

      """

      ## MAKE SURE OF 1 DRIVE DEGRADE CASE: Check if it's doable
      if len(raid_os.md_device_candidates)-len(raid_os.md_devices)!=1: 
         self.exitId+=1
         return

      
      self.device_to_replace=''
      for dev in raid_os.md_device_candidates:
         if dev in raid_os.md_devices: continue
         self.device_to_replace="/dev/%s"%dev
         break

      self.partner_device="/dev/%s" % raid_os.md_devices[0]      
      ### NEED TO MAKE SURE mdlevel is correct for legacy legacy like 411,511,etc
      if CONFIG.CentOS_version ==5 and raid_os.appliance.mdlevel==10: # only for CentOS5...having complicated RAID10
         for md in raid_os.mdraids:
            if "_" in md.statusBit:
               self.partner_device="/dev/%s" % (md.devices[0])[0:3]

      if CONFIG.verbose=="RAID_OS_SelfHeal":
         print self.partner_device, self.device_to_replace

      power_on_hours=get_nth_word("/usr/sbin/smartctl -A %s | grep 'Power_On_Hours'" % self.device_to_replace,9)
      if float(power_on_hours)/24. > 10.0 and not CONFIG.force_swrebuild:  # needs to be less than 10 day
         self.exitId+=2
         return

      
      isGPT=False
      command="/sbin/parted %s print free | grep -E 'Partition Table: gpt' | /usr/bin/wc -l" % self.partner_device
      if int(get_first_word(command)):
         isGPT=True

      self.isGPT=isGPT
      
      if not isGPT:
         self.exitId+=4
         return

      command="/sbin/sfdisk -d %s 2>&1 | grep -E 'unrecognized partition table type' | /usr/bin/wc -l" % self.device_to_replace
      if isGPT:
         command="/sbin/parted %s print 2>&1 | grep -E 'unrecognised disk label' | /usr/bin/wc -l" % self.device_to_replace

      if not int(get_first_word(command)):
         self.exitId+=8
         return
      #TODO: Currently, sfdisk (msdos) is out of picture. Maybe in future we like to include it?
         
      self.isGPT=isGPT
      if not isGPT:
         self.exitId+=8

         # when there are four empty partition, it's not doable in case of msdos partition
         #command="/sbin/sfdisk -d $disk 2>/dev/null | grep -E 'start.* 0.*size.* 0.*Id.* 0' | /usr/bin/wc -l"
         #if int(get_first_word(command))==4:
         #   self.exitId+=8
         
            

   def __repr__(self):
      return "%d   %s" %(self.exitId, self.device_to_replace)
      

   def write_table(self):

      import subprocess
      exitId=0

      if self.isGPT:
      
         command=\
"""
echo | /usr/sbin/gdisk %s << EOF
x
u
%s
y
q
EOF
""" % (self.partner_device,self.device_to_replace)

         if CONFIG.verbose=="RAID_OS_SelfHeal":
            print command
         
         #print_list(lines)
         #http://stackoverflow.com/questions/2502833/store-output-of-subprocess-popen-call-in-a-string
         #dd if=/dev/zero of=/dev/sda bs=512 count=10240
         p=subprocess.Popen(command, stdout=subprocess.PIPE,shell=True)
         out,err=p.communicate()

         if not "successfully" in out:
            exitId+=1

         else:
            command=\
"""
echo | /usr/sbin/gdisk %s << EOF
x
g
R
w
y
EOF
""" % (self.device_to_replace)
            if CONFIG.verbose=="RAID_OS_SelfHeal":
               print command

            p=subprocess.Popen(command, stdout=subprocess.PIPE,shell=True)
            out,err=p.communicate()
         
            if not "successfully" in out:
               exitId+=2
            else:
               os.system("sleep 5")  # Require giving time to subdevice created so that it's not rejected during the process


      else:  # not isGPT

         ## TODO: NEED TO ADD
         #command="|/sbin/sfdisk %s --force" % self.device_to_replace
         return 4
               
      return exitId
