#!/bin/bash

# Device class information:  https://pci-ids.ucw.cz/read/PD/
#   0x01     Mass storage controller
#   0x0104        RAID bus controller
#   0x0106        SATA controller
#   0x010601           AHCI 1.0
#   0x0107        Serial Attached SCSI controller
#   0x0180        Mass storage controller (eSATA on R-610, SiI 3512)

#   0x0c     Serial bus controller
#   0x0c03        USB controller
#   0x0c0300          UHCI pgm interface
#   0x0c0310          OHCI pgm interface
#   0x0c0320          EHCI pgm interface (Unitrends USB Stick)
#   0x0c0330          XHCI pgm interface
#   0x0c0380          Unspecified interface
#   0x0c03fe          USB device  

# List of allowed PCI controller type (see the comment at the top)
#   0x0104   RAID bus controller            ==> 3ware single in legacy 2U-,3U-,5U-   
#   0x0106   SATA controller                ==> OnBoard SATA
#   0x0107   Serial Attached SCSI controller==> MiniSAS controller
#   0x0180   Mass storage controller        ==>eSATA
#   0x0c03   USB controller                 ==>USB       
#[[ $class =~ ^(0x0104|0x0106|0x0107|0x0180|0x0c03) ]] || return 1 

scanDevices() {
    dev="$1"
    # Don't get the properties multiple times for the same device
    #[ "$dev" = "$lastDevice" ] && return 0
    
    # Don't leave stale values in the globals
    size=
    model=
    vendor=
    class=
    
    # Traverse up the device path, filling in properties as we go.  Trim the whitespace.
    sedscript='s/^[[:space:]]*//g; s/[[:space:]]*$//g'
    # CentOS5: /sys/devices/pci0000:00/0000:00:1f.2/host1/target1:0:0/1:0:0:0/block:sdc
    #    startDir=`find /sys/devices -name block:$1 2>/dev/null`
    # CentOS6: /sys/devices/pci0000:00/0000:00:1f.2/host3/target3:0:0/3:0:0:0/block/sdc
    #    startDir=`find /sys/devices -name *$1 2>/dev/null`
    
    #sometimes the "find" finds two paths and the next dirname fails like
    # find /sys/devices -name *sdg
    # /sys/devices/virtual/block/dm-1/slaves/sdg
    # /sys/devices/pci0000:80/0000:80:01.0/0000:81:00.0/0000:82:00.0/host14/target14:0:0/14:0:0:0/block/sdg
    startDir=`find /sys/devices -name *$dev | grep host 2>/dev/null`
    if [ -z "$startDir" ]; then
        exit
    fi
    #devdir=`echo "$startDir" | xargs dirname`
    
    cd $startDir
    while [[ `pwd` =~ ^/sys/devices ]]; do    # [[ "" ]] does not work in CentOS6
        [ -z "$size" ] && [ -f "size" ] && size=`cat size | sed "$sedscript"`
        [ -z "$model" ] && [ -f "model" ] && model=`cat model | sed "$sedscript"`
        [ -z "$vendor" ] && [ -f "vendor" ] && vendor=`cat vendor | sed "$sedscript"`
        [ -z "$class" ] && [ -f "class" ] && class=`cat class | sed "$sedscript"`
        cd ..
    done

    #lastDevice="$dev"

    echo $size
    echo $model
    echo $vendor
    echo $class
}


case $1 in
    --device_scan)
        scanDevices $2
        ;;

    *)
        echo "--device_scan sda"
        ;;
esac
